# -*- coding: utf-8 -*-

"""Top-level package for ganttify."""
import altair as alt
import pandas as pd

DEFAULT_COLOR_SCALE = alt.Scale(
    domain=[
        'queue',
        'success',
        'fail',
    ],
    range=['gray', 'blue', 'red'])

DEFAULT_LABEL_LIMIT = 1000
DEFAULT_WIDTH = 1000
DEFAULT_TIMESCALE = 'hoursminutes'

DEFAULT_YAXIS = alt.Axis(
    ticks=False,
    minExtent=60,
    domain=False)


def process_data(data):
    arr = []
    # we have to set an additional attribute so we can preserve
    # the sort order when showing the chart
    for i, entry in enumerate(data):
        status = 'success'
        if entry['status'].lower() != 'success':
            status = 'fail'

        if 'queue' in entry:
            arr.append({
                'label': entry['label'],
                'type': 'queue',
                'start': pd.to_datetime(entry['queue']),
                'end': pd.to_datetime(entry['start']),
                'idx': i,
            })

        arr.append({
            'label': entry['label'],
            'type': status,
            'start': pd.to_datetime(entry['start']),
            'end': pd.to_datetime(entry['end']),
            'idx': i,
        })

    return pd.DataFrame(arr)


def make_chart(
    data,
    outfile,
    color_scale=DEFAULT_COLOR_SCALE,
    labelLimit=DEFAULT_LABEL_LIMIT,
    width=DEFAULT_WIDTH,
    timescale=DEFAULT_TIMESCALE,
    yaxis=DEFAULT_YAXIS,
):
    df = process_data(data)

    chart = alt.Chart(df) \
        .mark_bar() \
        .configure_axisY(labelLimit=labelLimit) \
        .configure_view(width=width) \
        .encode(
            # we're slightly abusing the titles here
            x=alt.X(
                '{}(start):T'.format(timescale),
                title='Time'
            ),
            x2=alt.X(
                '{}(end):T'.format(timescale),
                title='Duration'
            ),
            y=alt.Y(
                'label:N',
                axis=yaxis,
                sort=alt.EncodingSortField(field='idx', op='sum', order='ascending'),
                title=None
            ),
            color=alt.Color(
                'type:N',
                legend=alt.Legend(title='segment'),
                scale=color_scale,
            ))

    chart.save(outfile)



import pytest

import ganttify


@pytest.fixture
def data():
    return [
        {
            'queue': '2019-04-02T16:51:15-05:00',
            'start': '2019-04-02T16:51:30-05:00',
            'end': '2019-04-02T16:52:22-05:00',
            'status': 'success',
            'label': 'foo1',
        },
        {
            'queue': '2019-04-02T16:51:15-05:00',
            'start': '2019-04-02T16:53:00-05:00',
            'end': '2019-04-02T16:53:44-05:00',
            'status': 'failure',
            'label': 'foo2',
        },
        {
            'start': '2019-04-02T16:51:33-05:00',
            'end': '2019-04-02T16:52:33-05:00',
            'status': 'success',
            'label': 'foo3',
        },
    ]


def describe_process_data():
    @pytest.fixture(autouse=True)
    def mocked_data_frame(mocker):
        patched = mocker.patch('ganttify.pd.DataFrame', autospec=True)
        return patched

    def creates_data_frmes_for_the_data(data, mocked_data_frame):
        result = ganttify.process_data(data)
        expected = []

        for i, entry in enumerate(data):
            status = 'success'
            if entry['status'].lower() != 'success':
                status = 'fail'

            if 'queue' in entry:
                expected.append({
                    'label': entry['label'],
                    'type': 'queue',
                    'start': ganttify.pd.to_datetime(entry['queue']),
                    'end': ganttify.pd.to_datetime(entry['start']),
                    'idx': i,
                })

            expected.append({
                'label': entry['label'],
                'type': status,
                'start': ganttify.pd.to_datetime(entry['start']),
                'end': ganttify.pd.to_datetime(entry['end']),
                'idx': i,
            })

        # sanity check to ensure the above does what we expected
        assert len(expected) == len(data) * 2 - 1

        assert result == mocked_data_frame.return_value
        mocked_data_frame.assert_called_once_with(expected)


def describe_make_chart():
    @pytest.fixture
    def outfile_name():
        return 'mychart.html'

    @pytest.fixture(autouse=True)
    def mocked_process_data(mocker):
        patched = mocker.patch('ganttify.process_data', autospec=True)
        return patched

    # so this is pretty crappy, but it's to be expected
    # given the way the altair api is
    def creates_the_chart(mocker, data, outfile_name, mocked_process_data):
        mocked_chart = mocker.patch('ganttify.alt.Chart', autospec=True)

        ganttify.make_chart(data, outfile_name)

        mocked_process_data.assert_called_once_with(data)
        mocked_chart.assert_called_once_with(
            ganttify.process_data(data))

        target = mocked_chart.return_value
        target.mark_bar.assert_called_once_with()

        target = target.mark_bar.return_value
        target.configure_axisY.assert_called_once_with(
            labelLimit=ganttify.DEFAULT_LABEL_LIMIT)

        target = target.configure_axisY.return_value
        target.configure_view.assert_called_once_with(
            width=ganttify.DEFAULT_WIDTH)

        target = target.configure_view.return_value
        target.encode.assert_called_once_with(
            x=ganttify.alt.X(
                '{}(start):T'.format(ganttify.DEFAULT_TIMESCALE),
                title='Time'
            ),
            x2=ganttify.alt.X(
                '{}(end):T'.format(ganttify.DEFAULT_TIMESCALE),
                title='Duration'
            ),
            y=ganttify.alt.Y(
                'label:N',
                axis=ganttify.DEFAULT_YAXIS,
                sort=ganttify.alt.EncodingSortField(
                    field='idx',
                    op='sum',
                    order='ascending'
                ),
                title=None
            ),
            color=ganttify.alt.Color(
                'type:N',
                legend=ganttify.alt.Legend(title='segment'),
                scale=ganttify.DEFAULT_COLOR_SCALE,
            ))

        target = target.encode.return_value
        target.save.assert_called_once_with(outfile_name)



import json
import os
import pytest

from click.testing import CliRunner

import ganttify
import ganttify.cli as cli


@pytest.fixture
def data():
    return [
        {
            'queue': '2019-04-02T16:51:15-05:00',
            'start': '2019-04-02T16:51:30-05:00',
            'end': '2019-04-02T16:52:22-05:00',
            'status': 'success',
            'label': 'foo1',
        },
        {
            'queue': '2019-04-02T16:51:15-05:00',
            'start': '2019-04-02T16:53:00-05:00',
            'end': '2019-04-02T16:53:44-05:00',
            'status': 'failure',
            'label': 'foo2',
        },
        {
            'start': '2019-04-02T16:51:33-05:00',
            'end': '2019-04-02T16:52:33-05:00',
            'status': 'success',
            'label': 'foo3',
        },
    ]


@pytest.fixture
def runner():
    return CliRunner()


def _invoke(runner, args):
    return runner.invoke(
        cli.cli,
        args,
        catch_exceptions=False)


# so these are actually integration tests, but they're fine thanks to tmpdir
def describe_cli():
    def makes_a_chart(mocker, runner, data, tmpdir):
        with tmpdir.as_cwd():
            # sanity check
            assert not os.path.exists(str(tmpdir.join('chart.html')))

            spy = mocker.patch('ganttify.make_chart', wraps=ganttify.make_chart)

            args = [json.dumps(data)]
            result = _invoke(runner, args)
            assert result.exit_code == 0

            spy.assert_called_once_with(
                data,
                'chart.html',
                labelLimit=ganttify.DEFAULT_LABEL_LIMIT,
                width=ganttify.DEFAULT_WIDTH,
                timescale=ganttify.DEFAULT_TIMESCALE)

            assert os.path.exists(str(tmpdir.join('chart.html')))

    def allows_overrides(mocker, runner, data, tmpdir):
        with tmpdir.as_cwd():
            # sanity check
            assert not os.path.exists(str(tmpdir.join('foo.html')))

            spy = mocker.patch('ganttify.make_chart', wraps=ganttify.make_chart)

            args = ['-o', 'foo.html', '-w', 11, '-l', 123, '-t', 'hours', json.dumps(data)]
            result = _invoke(runner, args)
            assert result.exit_code == 0

            spy.assert_called_once_with(
                data,
                'foo.html',
                labelLimit=123,
                width=11,
                timescale='hours')

            assert os.path.exists(str(tmpdir.join('foo.html')))

    def allows_for_data_to_be_a_file(mocker, runner, data, tmpdir):
        with tmpdir.as_cwd():
            # sanity check
            assert not os.path.exists(str(tmpdir.join('chart.html')))

            spy = mocker.patch('ganttify.make_chart', wraps=ganttify.make_chart)

            with open('datafile.json', 'w') as f:
                json.dump(data, f)

            args = ['datafile.json']
            result = _invoke(runner, args)
            assert result.exit_code == 0

            spy.assert_called_once_with(
                data,
                'chart.html',
                labelLimit=ganttify.DEFAULT_LABEL_LIMIT,
                width=ganttify.DEFAULT_WIDTH,
                timescale=ganttify.DEFAULT_TIMESCALE)

            assert os.path.exists(str(tmpdir.join('chart.html')))



# -*- coding: utf-8 -*-

"""CLI entrypoint for ganttify."""
import sys

from ganttify import cli


def main(args=None):
    if args is None:
        args = sys.argv[1:]

    # potentially modify this to a more complicated context object if necessary
    cli.cli()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

"""Console script for ganttify."""
import click
import json
import os

import ganttify


# allow -h as well as --help
CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])

TIMESCALE_CHOICES = [
    'date',
    'day',
    'hours',
    'hoursminutes',
    'hoursminutesseconds',
    'milliseconds'
    'minutes',
    'minutesseconds',
    'month',
    'monthdate',
    'quarter',
    'quartermonth',
    'seconds',
    'secondsmilliseconds',
    'year',
    'yearmonth',
    'yearmonthdate',
    'yearmonthdatehours',
    'yearmonthdatehoursminutes',
    'yearmonthdatehoursseconds',
    'yearquarter',
    'yearquartermonth',
]


@click.command(context_settings=CONTEXT_SETTINGS, name='ganttify')
@click.option(
    '-o',
    '--outfile',
    default='chart.html',
    help='The output file. Should be .html unless you have the prerequisites for other file types.'
)
@click.option(
    '-w',
    '--width',
    type=int,
    default=ganttify.DEFAULT_WIDTH,
    help='The width of the chart in pixels'
)
@click.option(
    '-l',
    '--label-limit',
    type=int,
    default=ganttify.DEFAULT_LABEL_LIMIT,
    help='The label length limit in pixels'
)
@click.option(
    '-t',
    '--timescale',
    type=click.Choice(TIMESCALE_CHOICES),
    default=ganttify.DEFAULT_TIMESCALE,
    help='The time units to use according to the Altair docs'
)
@click.argument('data', required=True)
def cli(data, outfile, width, label_limit, timescale):
    """This is a thin wrapper around the Altair python visualization library
    specifically for creating Gantt charts of CI pipeline results given data in
    the following JSON format:

    \b
    [
        {
            'queue': 'OPTIONAL - pandas parsable timestamp',
            'start': 'REQUIRED - pandas parsable timestamp',
            'end': 'REQUIRED - pandas parsable timestamp',
            'status': 'REQUIRED - any string other than "success" is interpreted as a failure',
            'label': 'REQUIRED - label string',
        },
        ...
    ]

    This satisfies my own use cases, use at your own risk.

    For more control, recommend importing the ganttify module and invoking
    make_chart from a script.

    The data argument here can be a json blob or a filename
    """
    try:
        data = json.loads(data)
    except ValueError:
        # this is a bit of a dumb sanity check, but it will probably suffice
        if '[' not in data and os.path.exists(data):
            with open(data) as f:
                data = json.load(f)
        else:
            raise

    ganttify.make_chart(
        data,
        outfile,
        labelLimit=label_limit,
        width=width,
        timescale=timescale)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages
from os import path
import sys


cur_dir = path.abspath(path.dirname(__file__))

if sys.version_info[0] < 3:
    with open(path.join(cur_dir, 'README.md')) as f:
        long_description = f.read()
else:
    with open(path.join(cur_dir, 'README.md'), encoding='utf-8') as f:
        long_description = f.read()

requirements = [
    'altair>=2.4.1,<3',
    'Click>=7.0',
    'pandas>=0.24.2',
]

setup_requirements = []

setup(
    author="Matt Chun-Lum",
    description="Turn test results into a Gantt chart",
    long_description=long_description,
    long_description_content_type='text/markdown',
    entry_points={
        'console_scripts': [
            'ganttify=ganttify.__main__:main'
        ],
    },
    install_requires=requirements,
    include_package_data=True,
    keywords='ganttify',
    name='ganttify',
    packages=find_packages(include=['ganttify']),
    setup_requires=setup_requirements,
    version='0.2.1',
    zip_safe=False,
)
